/* --------------------------------------------------------------
    Signed integers with unlimited range (version 1.2).

    http://www.imach.uran.ru/cbignum

    Copyright 1999-2006 by Raul N.Shakirov, IMach of RAS(UB).
    All Rights Reserved.

    Permission has been granted to copy, distribute and modify
    software in any context without fee, including a commercial
    application, provided that the aforesaid copyright statement
    is present here as well as exhaustive description of changes.

    THE SOFTWARE IS DISTRIBUTED "AS IS". NO WARRANTY OF ANY KIND
    IS EXPRESSED OR IMPLIED. YOU USE AT YOUR OWN RISK. THE AUTHOR
    WILL NOT BE LIABLE FOR DATA LOSS, DAMAGES, LOSS OF PROFITS OR
    ANY OTHER KIND OF LOSS WHILE USING OR MISUSING THIS SOFTWARE.
-------------------------------------------------------------- */
#include <assert.h>
#include <limits.h>
#include <stdlib.h>
#include "Cbignum.h"
#define BITS    (CHAR_BIT * sizeof (long))
static  int     log2_BITS = cLongExactLog2 (BITS);

#ifdef  _MSC_VER
#pragma auto_inline (off)
#endif/*_MSC_VER*/

//================================================
//      Service variables and methods.
//================================================

//      Service variables are either static or global to exclude
//      expenses for creation of temporary objects on execution
//      of time-critical methods. Memory, occupied for service
//      variables is not freed, so same variable may be used by
//      several methods, which must not call each others, except
//      for using of service variable for transferring of operands.

static  cBigNumber      a_;             // Buffer for base of power.
static  cBigNumber      b_;             // Buffer for degree of power, shift.
static  cBigNumber      num_;           // Buffer for power, shift, conversion.
cBigNumber cBigNumberLastDivMod;        // Remainder of last division.
cBigNumber cBigNumberLastRootRm;        // Remainder of last root.
exblockstack<long> cBigTemp::stack;     // Stack of temporary objects.

//================================================
//      Service functions.
//================================================

inline size_t size_max (size_t n1, size_t n2)
{
  return ((n1 > n2)? n1 : n2);
}
inline size_t size_min (size_t n1, size_t n2)
{
  return ((n1 < n2)? n1 : n2);
}

//================================================
//      Implementation of information methods.
//================================================

cBigNumber& cBigNumber::setbits     (const cBigNumber& b)
{
  size_t n = cBigNumberWords (EXPTRTYPE(b));
  set (n > 0? cULongBits ((unsigned long)
                (b.code()[--n] ^ (b.hiword()>>(BITS-1)))): 0);
  return addmul ((long)n, BITS);
}

cBigNumber& cBigNumber::setbits     (long b)
{
  return set (cLongBits (b));
}

cBigNumber& cBigNumber::setexbits   (const cBigNumber& b)
{
  size_t n = cBigNumberExWords (EXPTRTYPE(b));
  set (cLongBits (b.code()[n - (b.length() == 0)]));
  return addmul ((long)n, BITS);
}

cBigNumber& cBigNumber::setexbits   (long b)
{
  return set (cLongExBits (b));
}

//================================================
//      Implementation of service conversions
//================================================

cBigNumber& cBigNumber::fit()
{
  if (length()==0) checkexpand (1);
  checkindex (cBigNumberFit (EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::tab()
{
  checkexpand (exmuladd (BITS, length(), BITS * 2 + 1));
  cBigNumberTab (EXPTRTYPE(*this));
  return *this;
}

cBigNumber& cBigNumber::gc()
{
  checkshrink (length());
  return *this;
}

cBigNumber& cBigNumber::pack()
{
  reallocate (length() + (length()==0) + 1);
  return *this;
}

//================================================
//      Implementation of unary operations.
//================================================

cBigNumber& cBigNumber::operator ++()
{
  checkexpand (length() + 1);
  checkindex (cBigNumberMInc (EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::operator --()
{
  checkexpand (length() + 1);
  checkindex (cBigNumberMDec (EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::mul2()
{
  checkexpand (length() + 1);
  checkindex (cBigNumberMMul2 (EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::div2()
{
  checkindex (cBigNumberMDiv2 (EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::pow2()
{
  checkexpand (length() * 2 + 2);
  cBigNumberMul (EXPTRTYPE(*this), EXPTRTYPE(*this), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::sqrt()
{
  swap (cBigNumberLastRootRm);
  checkexpand (cBigNumberLastRootRm.fit().length() / 2 + 1);
  if (cBigNumberLastRootRm.hiword() >= 0)
    cBigNumberMRmSqrt (EXPTRTYPE(cBigNumberLastRootRm), EXPTRTYPE(*this));
  else
    *this = 0;
  checkindex (length());
  return *this;
}

//================================================
//      Implementation of assign operations.
//================================================

cBigNumber& cBigNumber::setxor (const cBigNumber& a, const cBigNumber& b)
{
  checkindex ((a.length() >= b.length())?
    (checkexpand (a.length()),
     cBigNumberXor (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this))):
    (checkexpand (b.length()),
     cBigNumberXor (EXPTRTYPE(b), EXPTRTYPE(a), EXPTRTYPE(*this))));
  return *this;
}

cBigNumber& cBigNumber::setxor (const cBigNumber& a, long b)
{
  checkexpand (a.length());
  checkindex (cBigNumberXor (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setxor (long a, const cBigNumber& b)
{
  checkexpand (b.length());
  checkindex (cBigNumberXor (EXPTRTYPE(b), EXPTRTYPE(_cBigLong(a)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setxor (long a, long b)
{
  return *this = a ^ b;
}

cBigNumber& cBigNumber::setand (const cBigNumber& a, const cBigNumber& b)
{
  checkindex ((a.length() >= b.length())?
    (checkexpand (a.length()),
     cBigNumberAnd (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this))):
    (checkexpand (b.length()),
     cBigNumberAnd (EXPTRTYPE(b), EXPTRTYPE(a), EXPTRTYPE(*this))));
  return *this;
}

cBigNumber& cBigNumber::setand (const cBigNumber& a, long b)
{
  checkexpand (a.length());
  checkindex (cBigNumberAnd (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setand (long a, const cBigNumber& b)
{
  checkexpand (b.length());
  checkindex (cBigNumberAnd (EXPTRTYPE(b), EXPTRTYPE(_cBigLong(a)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setand (long a, long b)
{
  return *this = a & b;
}

cBigNumber& cBigNumber::setor  (const cBigNumber& a, const cBigNumber& b)
{
  checkindex ((a.length() >= b.length())?
    (checkexpand (a.length()),
     cBigNumberOr (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this))):
    (checkexpand (b.length()),
     cBigNumberOr (EXPTRTYPE(b), EXPTRTYPE(a), EXPTRTYPE(*this))));
  return *this;
}

cBigNumber& cBigNumber::setor  (const cBigNumber& a, long b)
{
  checkexpand (a.length());
  checkindex (cBigNumberOr (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)),
                            EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setor  (long a, const cBigNumber& b)
{
  checkexpand (b.length());
  checkindex (cBigNumberOr (EXPTRTYPE(b), EXPTRTYPE(_cBigLong(a)),
                            EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setor  (long a, long b)
{
  return *this = a | b;
}

cBigNumber& cBigNumber::setadd (const cBigNumber& a, const cBigNumber& b)
{
  checkindex ((a.length() >= b.length())?
    (checkexpand (a.length() + 1),
     cBigNumberAdd (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this))):
    (checkexpand (b.length() + 1),
     cBigNumberAdd (EXPTRTYPE(b), EXPTRTYPE(a), EXPTRTYPE(*this))));
  return *this;
}

cBigNumber& cBigNumber::setadd (const cBigNumber& a, long b)
{
  checkexpand (a.length() + 1);
  checkindex (cBigNumberAdd (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setadd (long a, const cBigNumber& b)
{
  checkexpand (b.length() + 1);
  checkindex (cBigNumberAdd (EXPTRTYPE(b), EXPTRTYPE(_cBigLong(a)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setadd (long a, long b)
{
  checkexpand (2);
  checkindex (cBigNumberAdd (EXPTRTYPE(_cBigLong(a)), EXPTRTYPE(_cBigLong(b)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setsub (const cBigNumber& a, const cBigNumber& b)
{
  checkindex ((a.length() >= b.length())?
    (checkexpand (a.length() + 1),
     cBigNumberSub  (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this))):
    (checkexpand (b.length() + 1),
     cBigNumberSubS (EXPTRTYPE(b), EXPTRTYPE(a), EXPTRTYPE(*this))));
  return *this;
}

cBigNumber& cBigNumber::setsub (const cBigNumber& a, long b)
{
  checkexpand (a.length() + 1);
  checkindex (cBigNumberSub (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setsub (long a, const cBigNumber& b)
{
  checkexpand (b.length() + 1);
  checkindex (cBigNumberSubS(EXPTRTYPE(b), EXPTRTYPE(_cBigLong(a)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setsub (long a, long b)
{
  checkexpand (2);
  checkindex (cBigNumberSub (EXPTRTYPE(_cBigLong(a)), EXPTRTYPE(_cBigLong(b)),
                             EXPTRTYPE(*this)));
  return *this;
}

cBigNumber& cBigNumber::setmul (const cBigNumber& a, const cBigNumber& b)
{
  size_t la = a.length(), lb = b.length();
  checkexpand (la + lb + 2);
  if (la >= lb)
        cBigNumberMul (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this));
  else  cBigNumberMul (EXPTRTYPE(b), EXPTRTYPE(a), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setmul (const cBigNumber& a, long b)
{
  checkexpand (a.length() + 3);
  cBigNumberMul (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setmul (long a, const cBigNumber& b)
{
  checkexpand (b.length() + 3);
  cBigNumberMul (EXPTRTYPE(b), EXPTRTYPE(_cBigLong(a)), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setmul (long a, long b)
{
  checkexpand (4);
  cBigNumberMul (EXPTRTYPE(_cBigLong(a)), EXPTRTYPE(_cBigLong(b)),
                 EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setdiv (const cBigNumber& a, const cBigNumber& b)
{
  return setdivmod (cBigNumberLastDivMod = a, b);
}

cBigNumber& cBigNumber::setdiv (const cBigNumber& a, long b)
{
  return setdivmod (cBigNumberLastDivMod = a, b);
}

cBigNumber& cBigNumber::setdiv (long a, const cBigNumber& b)
{
  return setdivmod (cBigNumberLastDivMod = a, b);
}

cBigNumber& cBigNumber::setdiv (long a, long b)
{
  return setdivmod (cBigNumberLastDivMod = a, b);
}

cBigNumber& cBigNumber::setmod (const cBigNumber& a, const cBigNumber& b)
{
  return (*this = a) %= b;
}

cBigNumber& cBigNumber::setmod (const cBigNumber& a, long b)
{
  return (*this = a) %= b;
}

cBigNumber& cBigNumber::setmod (long a, const cBigNumber& b)
{
  return (*this = a) %= b;
}

cBigNumber& cBigNumber::setmod (long a, long b)
{
  return (*this = a) %= b;
}

//================================================
//      Implementation of shifts.
//================================================

cBigNumber& cBigNumber::operator <<= (const cBigNumber& b)
{
  if (b.hiword() < 0) return (*this >>= (-b));
  num_.setdivmod (b_ = b, BITS);
  size_t n = (size_t)b_.loword();               // Shift in bits.
  fit();
  checkexpand ((num_.length() <= 1 &&
               (unsigned long)num_.loword() < UINT_MAX - 2 - length())?
                      (size_t)num_.loword() + length() + 2: UINT_MAX);
  for (; n > 0; --n) cBigNumberMMul2 (EXPTRTYPE(*this));
  cBigNumberCopyShl (EXPTRTYPE(*this), (size_t)num_.loword(),
                     EXPTRTYPE(*this));
  checkindex (length());
  EXDEBUG (num_ = 0);                           // Debug cleaning.
  return *this;
}

cBigNumber& cBigNumber::operator <<= (long b)
{
  if (b < 0) return (*this >>= (-b));
  int n = (int)(b % BITS); b /= BITS;
  fit();
  checkexpand ((unsigned long)b < UINT_MAX - 2 - length()?
                      (size_t)b + length() + 2: UINT_MAX);
  for (; n > 0; --n) cBigNumberMMul2 (EXPTRTYPE(*this));
  cBigNumberCopyShl (EXPTRTYPE(*this), (size_t)b, EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setshl (const cBigNumber& a, const cBigNumber& b)
{
  *this = a; return *this <<= b;
}

cBigNumber& cBigNumber::setshl (const cBigNumber& a, long b)
{
  *this = a; return *this <<= b;
}

cBigNumber& cBigNumber::setshl (long a, const cBigNumber& b)
{
  *this = a; return *this <<= b;
}

cBigNumber& cBigNumber::setshl (long a, long b)
{
  *this = a; return *this <<= b;
}

cBigNumber& cBigNumber::operator >>= (const cBigNumber& b)
{
  if (b.hiword() < 0) return (*this <<= (-b));
  num_.setdivmod (b_ = b, BITS);
  size_t n = (size_t)b_.loword();               // Shift in bits.
  fit();
  for (; n > 0; --n)  cBigNumberMDiv2 (EXPTRTYPE(*this));
  cBigNumberCopyShr (EXPTRTYPE(*this),
                     (num_.length() <= 1 &&
                     (unsigned long)num_.loword() <= UINT_MAX)?
                            (size_t)num_.loword(): UINT_MAX,
                     EXPTRTYPE(*this));
  checkindex (length());
  EXDEBUG (num_ = 0);                           // Debug cleaning.
  return *this;
}

cBigNumber& cBigNumber::operator >>= (long b)
{
  if (b < 0) return (*this <<= (-b));
  size_t n = (size_t) b % BITS; b /= BITS;
  fit();
  for (; n > 0; --n) cBigNumberMDiv2 (EXPTRTYPE(*this));
  cBigNumberCopyShr (EXPTRTYPE(*this),
                     (unsigned long)b <= UINT_MAX?
                            (size_t)b: UINT_MAX,
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setshr (const cBigNumber& a, const cBigNumber& b)
{
  *this = a; return *this >>= b;
}

cBigNumber& cBigNumber::setshr (const cBigNumber& a, long b)
{
  *this = a; return *this >>= b;
}

cBigNumber& cBigNumber::setshr (long a, const cBigNumber& b)
{
  *this = a; return *this >>= b;
}

cBigNumber& cBigNumber::setshr (long a, long b)
{
  *this = a; return *this >>= b;
}

//================================================
//      Implementation of power.
//================================================

cBigNumber& cBigNumber::setpow_()               // Private method.
{
  if (b_.hiword() < 0)                          // Negative degree.
  {
    a_.setdiv (1,a_); b_.neg();
  }
  num_.setbits (a_); num_ *= b_;                // Estimate for size of result.
  if (log2_BITS >= 0)
    num_ >>= log2_BITS;                         // Optimized division.
  else
    num_.setdivmod (*this = num_, BITS);        // Generic division.
  size_t n = ((num_.length() <= 1 &&
              (unsigned long)num_.loword() <= UINT_MAX - 3)?
                    ((size_t)num_.loword() + 3): UINT_MAX);
  checkexpand (n); a_.checkexpand (n);
  cBigNumberPow (EXPTRTYPE(a_), EXPTRTYPE(b_), EXPTRTYPE(*this));
  checkindex (length());
  EXDEBUG (a_ = 0);                             // Debug cleaning.
  EXDEBUG (b_ = 0);                             // Debug cleaning.
  EXDEBUG (num_ = 0);                           // Debug cleaning.
  return *this;
}

cBigNumber& cBigNumber::setpow (const cBigNumber& a, const cBigNumber& b)
{
  a_ = a; b_ = b; return setpow_();
}

cBigNumber& cBigNumber::setpow (const cBigNumber& a, long b)
{
  a_ = a; b_ = b; return setpow_();
}

cBigNumber& cBigNumber::setpow (long a, const cBigNumber& b)
{
  a_ = a; b_ = b; return setpow_();
}

cBigNumber& cBigNumber::setpow (long a, long b)
{
  a_ = a; b_ = b; return setpow_();
}

//      Power by unlimited module.

cBigNumber& cBigNumber::setpowmod_ (const cBigNumber &mod)
{
  if (b_.hiword() < 0)                          // Negative degree.
  {
    a_.setdiv (1,a_); b_.neg();
  }
  size_t n = mod.length() * 2 + 1;              // Size of buffer.
  checkexpand (n); a_.checkexpand (size_max (a_.length() + 1, n));
  cBigNumberPowMod (EXPTRTYPE(a_), EXPTRTYPE(b_), EXPTRTYPE(mod),
                    EXPTRTYPE(*this));
  checkindex (length());
  EXDEBUG (a_ = 0);                             // Debug cleaning.
  EXDEBUG (b_ = 0);                             // Debug cleaning.
  return *this;
}

cBigNumber& cBigNumber::setpowmod (const cBigNumber& a, const cBigNumber& b,
                                   const cBigNumber& mod)
{
  a_ = a; b_ = b; return setpowmod_ (mod);
}

cBigNumber& cBigNumber::setpowmod (const cBigNumber& a, long b,
                                   const cBigNumber& mod)
{
  a_ = a; b_ = b; return setpowmod_ (mod);
}

cBigNumber& cBigNumber::setpowmod (long a, const cBigNumber& b,
                                   const cBigNumber& mod)
{
  a_ = a; b_ = b; return setpowmod_ (mod);
}

cBigNumber& cBigNumber::setpowmod (long a, long b,
                                   const cBigNumber& mod)
{
  a_ = a; b_ = b; return setpowmod_ (mod);
}

//      Power by long module.

cBigNumber& cBigNumber::setpowmod_ (long mod)
{
  if (b_.hiword() < 0)                          // Negative degree.
  {
    a_.setdiv (1,a_); b_.neg();
  }
  checkexpand (3); a_.checkexpand (size_max (a_.length() + 1, 3));
  cBigNumberPowMod (EXPTRTYPE(a_), EXPTRTYPE(b_), EXPTRTYPE(_cBigLong(mod)),
                    EXPTRTYPE(*this));
  checkindex (length());
  EXDEBUG (a_ = 0);                             // Debug cleaning.
  EXDEBUG (b_ = 0);                             // Debug cleaning.
  return *this;
}

cBigNumber& cBigNumber::setpowmod (const cBigNumber& a, const cBigNumber& b,
                                   long mod)
{
  a_ = a; b_ = b; return setpowmod_ (mod);
}

cBigNumber& cBigNumber::setpowmod (const cBigNumber& a, long b,
                                   long mod)
{
  a_ = a; b_ = b; return setpowmod_ (mod);
}

cBigNumber& cBigNumber::setpowmod (long a, const cBigNumber& b,
                                   long mod)
{
  a_ = a; b_ = b; return setpowmod_ (mod);
}

cBigNumber& cBigNumber::setpowmod (long a, long b,
                                   long mod)
{
  a_ = a; b_ = b; return setpowmod_ (mod);
}

//================================================
//      Implementation of square root.
//================================================

cBigNumber& cBigNumber::setsqrt (const cBigNumber& b)
{
  cBigNumberLastRootRm = b;
  checkexpand (cBigNumberLastRootRm.fit().length() / 2 + 1);
  if (cBigNumberLastRootRm.hiword() >= 0)
    cBigNumberMRmSqrt (EXPTRTYPE(cBigNumberLastRootRm), EXPTRTYPE(*this));
  else
    *this = 0;
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setsqrt (long b)
{
  checkexpand (1);
  cBigNumberLastRootRm = b;
  if (b >= 0)
    cBigNumberMRmSqrt (EXPTRTYPE(cBigNumberLastRootRm), EXPTRTYPE(*this));
  else
    *this = 0;
  checkindex (length());
  return *this;
}

//================================================
//      Implementation of random generator.
//================================================

cBigNumber& cBigNumber::setrandom (unsigned long (*p)(), unsigned long lBits)
{
  checkexpand ((size_t)(lBits / BITS + 1));
  cBigNumberRandom (p, lBits, EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

//================================================
//      Implementation of accumulation.
//================================================

cBigNumber& cBigNumber::add (const cBigNumber& b, size_t k)
{
  checkexpand (size_max (length(), b.length() + k) + 1);
  cBigNumberMAddShl (EXPTRTYPE(*this), EXPTRTYPE(b), k);
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::add (long b, size_t k)
{
  checkexpand (size_max (length(), k + 1) + 1);
  cBigNumberMAddShl (EXPTRTYPE(*this), EXPTRTYPE(_cBigLong(b)), k);
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::sub (const cBigNumber& b, size_t k)
{
  checkexpand (size_max (length(), b.length() + k) + 1);
  cBigNumberMSubShl (EXPTRTYPE(*this), EXPTRTYPE(b), k);
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::sub (long b, size_t k)
{
  checkexpand (size_max (length(), k + 1) + 1);
  cBigNumberMSubShl (EXPTRTYPE(*this), EXPTRTYPE(_cBigLong(b)), k);
  checkindex (length());
  return *this;
}

//================================================
//      Implementation of combined operations.
//================================================

cBigNumber& cBigNumber::addmul (const cBigNumber& a, const cBigNumber& b)
{
  size_t la = a.length(),lb = b.length();
  checkexpand (size_max (length(), la + lb + 1) + 1);
  if (la >= lb)
        cBigNumberMAddMul (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this));
  else  cBigNumberMAddMul (EXPTRTYPE(b), EXPTRTYPE(a), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::addmul (const cBigNumber& a, long b)
{
  checkexpand (size_max (length(), a.length() + 2) + 1);
  cBigNumberMAddMul (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)),
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::addmul (long a, const cBigNumber& b)
{
  checkexpand (size_max (length(), b.length() + 2) + 1);
  cBigNumberMAddMul (EXPTRTYPE(b), EXPTRTYPE(_cBigLong(a)),
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::addmul (long a, long b)
{
  checkexpand (size_max (length(), 3) + 1);
  cBigNumberMAddMul (EXPTRTYPE(_cBigLong(a)), EXPTRTYPE(_cBigLong(b)),
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::submul (const cBigNumber& a, const cBigNumber& b)
{
  size_t la = a.length(),lb = b.length();
  checkexpand (size_max (length(), la + lb + 1) + 1);
  if (la >= lb)
        cBigNumberMSubMul (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this));
  else  cBigNumberMSubMul (EXPTRTYPE(b), EXPTRTYPE(a), EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::submul (const cBigNumber& a, long b)
{
  checkexpand (size_max (length(), a.length() + 2) + 1);
  cBigNumberMSubMul (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)),
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::submul (long a, const cBigNumber& b)
{
  checkexpand (size_max (length(), b.length() + 2) + 1);
  cBigNumberMSubMul (EXPTRTYPE(b), EXPTRTYPE(_cBigLong(a)),
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::submul (long a, long b)
{
  checkexpand (size_max (length(), 3) + 1);
  cBigNumberMSubMul (EXPTRTYPE(_cBigLong(a)), EXPTRTYPE(_cBigLong(b)),
                     EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setdivmod (cBigNumber& a, const cBigNumber& b)
{
  checkexpand (a.fit().length() + 2);
  cBigNumberMModDiv (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this));
  a.checkindex (a.length());
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setdivmod (cBigNumber& a, long b)
{
  checkexpand (a.fit().length() + 2);
  cBigNumberMModDiv (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)),
                     EXPTRTYPE(*this));
  a.checkindex (a.length());
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setsqrtrm (cBigNumber& a)
{
  checkexpand (a.fit().length() / 2 + 1);
  if (a.hiword() >= 0)
    cBigNumberMRmSqrt (EXPTRTYPE(a), EXPTRTYPE(*this));
  else
    *this = 0;
  a.checkindex (a.length());
  checkindex (length());
  return *this;
}

//================================================
//      Implementation of shift table operations.
//================================================

cBigNumber& cBigNumber::addmultab (const cBigNumber& a, const cBigNumber& b,
                                   size_t k)
{
  checkexpand (size_max (length(), a.length() + b.length() + k) + 1);
  assert (b.hiword() >= 0);
  cBigNumberMAddMulShlTab (EXPTRTYPE(a), EXPTRTYPE(b), k, EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::addmultab (const cBigNumber& a, long b, size_t k)
{
  checkexpand (size_max (length(), a.length() + k + 1) + 1);
  assert (b >= 0);
  cBigNumberMAddMulShlTab (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)), k,
                           EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::submultab (const cBigNumber& a, const cBigNumber& b,
                                   size_t k)
{
  checkexpand (size_max (length(), a.length() + b.length() + k) + 1);
  assert (b.hiword() >= 0);
  cBigNumberMSubMulShlTab (EXPTRTYPE(a), EXPTRTYPE(b), k, EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::submultab (const cBigNumber& a, long b, size_t k)
{
  checkexpand (size_max (length(), a.length() + k + 1) + 1);
  assert (b >= 0);
  cBigNumberMSubMulShlTab (EXPTRTYPE(a), EXPTRTYPE(_cBigLong(b)), k,
                           EXPTRTYPE(*this));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setdivtab (const cBigNumber& a, const cBigNumber& b)
{
  cBigNumberLastDivMod = a;
  checkexpand (cBigNumberLastDivMod.fit().length() + 2);
  cBigNumberMModDivTab (EXPTRTYPE(cBigNumberLastDivMod), EXPTRTYPE(b),
                        EXPTRTYPE(*this));
  cBigNumberLastDivMod.checkindex (cBigNumberLastDivMod.length());
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setdivtab (long a, const cBigNumber& b)
{
  cBigNumberLastDivMod = a;
  checkexpand (3);
  cBigNumberMModDivTab (EXPTRTYPE(cBigNumberLastDivMod), EXPTRTYPE(b),
                        EXPTRTYPE(*this));
  cBigNumberLastDivMod.checkindex (cBigNumberLastDivMod.length());
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setmodtab (const cBigNumber& a, const cBigNumber& b)
{
  *this = a; fit();
  cBigNumberMModTab (EXPTRTYPE(*this), EXPTRTYPE(b));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setmodtab (long a, const cBigNumber& b)
{
  *this = a;
  cBigNumberMModTab (EXPTRTYPE(*this), EXPTRTYPE(b));
  checkindex (length());
  return *this;
}

cBigNumber& cBigNumber::setdivmodtab (cBigNumber& a, const cBigNumber& b)
{
  checkexpand (a.fit().length() + 2);
  cBigNumberMModDivTab (EXPTRTYPE(a), EXPTRTYPE(b), EXPTRTYPE(*this));
  a.checkindex (a.length());
  checkindex (length());
  return *this;
}

//================================================
//      Basic tests for primality of numbers.
//================================================

//      Strong Probable Primarily test for base b
//      by Olivier Langlois <olanglois@sympatico.ca>
//      Returns:
//      0 - composite
//      2 - probably prime
//      http://primes.utm.edu/glossary/page.php?sort=StrongPRP

int     b_SPRP  (
        const cBigNumber& n,            // Number to test.
        const cBigNumber& b             // Base.
)
{
  if (n <= 3) return (n < 2? 0: 2);     // Check for small numbers.
  if ((n.loword() & 1) == 0) return 0;  // Check if number is odd.

//                                            a
//      Find t and a satisfying: n - 1 = t * 2 , t odd
//

  cBigNumber t = n;
  cBigNumber nMinus1 = --t;
  cBigNumber a = nMinus1.exbits();
  t >>= a;

//      We check the congruence class of      i
//                                       t * 2
//                                      b      % n
//      for each i from 0 to a - 1.
//      If any one is correct, then n passes.
//      Otherwise, n fails.

  cBigNumber test; test.setpowmod (b, t, n);

  if (test == 1 || test == nMinus1) return 2;

  if (a)
  {
    while (--a)
    {
      test.pow2(); test %= n; if (test == nMinus1) return 2;
    }
  }

  return 0;
}

//================================================
//      Messages.
//================================================

const char* cBigNumber::pszConverting   = "Converting";
const char* cBigNumber::pszFormatting   = "Formatting";

//================================================
//      Tables for conversion routines.
//================================================

//      Table for conversion of digits to symbols.

const char  cBigNumber::numtochar [2] [MAX_RADIX + 1] =
                       {{'0','1','2','3','4','5','6','7','8','9',
                         'a','b','c','d','e','f','x'},
                        {'0','1','2','3','4','5','6','7','8','9',
                         'A','B','C','D','E','F','X'}};

//      Table for conversion of symbols to digits, build by static SqrTab()

      char  cBigNumber::chartonum [UCHAR_MAX + 1] = { 0 };

//      Table of coefficients contains maximal integer power of type long
//      and number of symbols in string representation of the aforesaid
//      power for each radix.

const long  cBigNumber::maxradix[][4] = {{ 0L,  1},  //  0
                                         { 1L,  1},  //  1
                                { 1073741824L, 30},  //  2
                                { 1162261467L, 19},  //  3
                                { 1073741824L, 15},  //  4
                                { 1220703125L, 13},  //  5
                                {  362797056L, 11},  //  6
                                { 1977326743L, 11},  //  7
                                { 1073741824L, 10},  //  8
                                {  387420489L,  9},  //  9
                                { 1000000000L,  9},  // 10
                                {  214358881L,  8},  // 11
                                {  429981696L,  8},  // 12
                                {  815730721L,  8},  // 13
                                { 1475789056L,  8},  // 14
                                {  170859375L,  7},  // 15
                                {  268435456L,  7} };// 16

//      For optimization of input/output of unlimited numbers for
//      each radix array with results co consequent squaring of
//      coefficient is created. For each coefficient table of shifts
//      is generated. Maximal coefficient MAX_NP is determined by
//      the criteria of minimization of input/output time for numbers
//      of size 1,000,000 bit on processor with fast cache of size >=256K.

#define MAX_NP_IN   ((sizeof (size_t))==2? 2: 9)
#define MAX_NP_OUT  ((sizeof (size_t))==2? 2: 9)
#define MAX_NP      (MAX_NP_IN > MAX_NP_OUT? MAX_NP_IN: MAX_NP_OUT)

exarray<cBigNumber> cBigNumber::powradix [MAX_RADIX + 1];

//      Filling of conversion table and array of coefficients.
//      Table must contain low coefficient and its square.

size_t  cBigNumber::SqrTab (
                unsigned radix,         // Radix (2..16).
                size_t len,             // Number of words in number.
                size_t max_np           // Maximal coefficient.
)                                       // Returns required number of
{                                       // coefficients (0..MAX_NP).
  assert (radix >= 2);
  assert (radix <= MAX_RADIX);
  assert (max_np <= MAX_NP);
  static size_t npow [MAX_RADIX + 1];   // Number of coefficients.

  if (npow [radix] == 0)
  {
//      Conversion table.

    int i;
    for (i = 0; i <= UCHAR_MAX; i++)
    {
      chartonum [i] = MAX_RADIX;
    }
    for (i = 0; i < MAX_RADIX; i++)
    {
      chartonum [numtochar [0] [i]] = (char)i;
      chartonum [numtochar [1] [i]] = (char)i;
    }

//      The lowest coefficient and his square.

    powradix [radix] [0] = maxradix [radix] [0];
    powradix [radix] [0].tab();         // Table of shifts.
    powradix [radix] [1] = maxradix [radix] [0];
    powradix [radix] [1].pow2();        // Square of coefficient.
    powradix [radix] [1].tab();         // Table of shifts.
    npow [radix] = 2;                   // Count of coefficients.
  }

//      Higher coefficients, if required.

  size_t np = 0;                        // Index of coefficient.
  while (np < max_np && powradix [radix] [np].length() * 4 < len)
  {
    if (++np >= npow [radix])
    {
      powradix [radix] [np] = powradix [radix] [np - 1];
      powradix [radix] [np].pow2();     // Square of coefficient.
      powradix [radix] [np].tab();      // Table of shifts.
      ++ npow [radix];                  // Count of coefficients.
    }
  }
  assert (np <= max_np);
  return np;
}

//      Work buffers.

static cBigNumber           mod_;           // Module of division.
static exvector<cBigNumber> stk_(MAX_NP);   // Stack of intermediate numbers.
static exvector<int>       kstk_(MAX_NP);   // Stack of flags.

//================================================
//      Conversion of string to unlimited number.
//================================================

//      Conversion of char array to unsigned unlimited number of radix 2..16.

cBigNumber& cBigNumber::setunsign (     // Conversion from unsigned char array.
                const EXPTR(char) ps,   // Char array with digits.
                unsigned radix,         // Radix (2..16).
                unsigned width          // Number of chars in array.
            )
{
  assert (ps);
  assert (radix >= 2);
  assert (radix <= MAX_RADIX);

//      Input of number.
//
//      Initial (non-optimized algorithm looks as the following:
//      *this = 0;
//      for (; width > 0; --width)
//      {
//        *this *= radix;
//        *this += chartonum [*ps++];
//      }
//
//      In optimized algorithm we use method addmultab for
//      multiplication with accumulation. To reduce number of
//      long multiplications we use multiplication to coefficients
//      powradix, obtained by consequent squaring of radix.
//      Results of consequent multiplications with accumulation
//      are stored in num_ and stack of multipliers mul.
//      Initially stack is empty.
//
//      ALGORITHM
//      Multipliers of 0-th order are represented by long word
//      and are obtained by conversion of short subchains.
//      Multiplier of i-th order is obtained by addmultab as
//      accumulation of two consequent multipliers of order i-1,
//      first of which is stored in [i].
//      Result number is obtained by addmultab as consequent
//      accumulation of highest multiplier stored in num_.

  *this = 0;
  if (width != 0)
  {
    long lradix =       maxradix [radix] [0];
    int  nradix = (int) maxradix [radix] [1];
    int nr = (width - 1) / nradix;      // Number of full words (no more).
    int n = width - nr++ * nradix;      // Number of symbols in last word.

//      Filling of stack of coefficients powradix [radix].
//      Required number of coefficients is returned.

    size_t np = SqrTab (radix, nr, MAX_NP_IN);
    EXPTR(cBigNumber) powr = EXPTRTYPE(powradix [radix]);

//      Zero-filling of stack of multipliers.

    EXPTR(cBigNumber)  mul = EXPTRTYPE(stk_);   // Stack of multipliers.
    { for (size_t i = 1; i < np; i++) mul [i] = 0; }

//      Progress indication.

    cTTY cProgress (cBigNumberProgress);
    size_t numlength = nr * sizeof (long);
    if (numlength < cTTY_numscaleK) numlength = 0;
    if (numlength != 0 && np == MAX_NP_IN)
      cProgress.showScaled (cBigNumber::pszConverting, 0, (long)numlength);

//      Cycle on pair groups of nradix digits.

    long lhigh = 0;                     // The high word.
    if ((nr & 1) != 0) goto odd;        // The lower word (may be incomplete).
    do
    {
//      Conversion of high group of nradix digits.

      {
        long lb = 0;                    // Word.
        for(;;)
        {
          lb += chartonum [*ps++];
          assert (lb >= 0);
          if (--n == 0) break;
          lb *= radix;
        }
        n = nradix;
        lhigh = lb;                     // The high word.
      }
      --nr;                             // Have converted the group.

//      Conversion of lower group of nradix digits.

odd:  {
        long lb = 0;                    // Word.
        for(;;)
        {
          lb += chartonum [*ps++];
          assert (lb >= 0);
          if (--n == 0) break;
          lb *= radix;
        }
        n = nradix;
        num_ = lb;                      // Lower word.
      }
      --nr;                             // Have converted the group.

//      Calculation of multiplier of 1st order.

      num_.addmul (lhigh, lradix);      // Accumulation of words.
      assert (num_ >= 0);

//      Calculation of multipliers of 2nd and higher order.
//      and accumulation of the number.

      for (size_t i = 1;; i++)
      {
        if (i < np)                     // Calculation of multipliers.
        {
          if ((nr & (2<<i)-1) != 0)     // Odd multiplier
          {                             // is stored
            num_.swap (mul [i]);        // in the stack.
            break;
          }                             // Otherwise multiplier of
                                        // higher order is calculated.
          num_.addmultab (powr [i], mul [i]);
          assert (num_ >= 0);
        }
        else                            // Accumulation of the number.
        {
          if (numlength != 0 && i == MAX_NP_IN)
            cProgress.showScaled (cBigNumber::pszConverting,
              (long)numlength - nr * sizeof (long), (long)numlength);
          num_.addmultab (powr [i], *this);
          assert (num_ >= 0);
          num_.swap (*this);
          break;
        }
      }
    }
    while (nr != 0);
  }

//      Conversion finished.

  EXDEBUG (num_ = 0);                   // Debug clearing.
  return *this;
}

//      Conversion of string to signed unlimited number of radix 2..16.
//      If radix is not indicated (set to 0) then radix is set to
//      10, 8 or 16 depending of prefixes 0, 0x, 0X.

cBigNumber& cBigNumber::set (           // Conversion from string.
                const char* psz,        // String.
                unsigned radix          // Radix (2..16) or 0.
            )                           // 0 - by C conventions.
{
  assert (psz);
  assert (radix != 1);
  assert (radix <= MAX_RADIX);
  int sign = 0;                         // Sign of number.
  unsigned c = *psz++;                  // Current symbol.

//      Recognition of sign.

  switch (c)
  {
    case '-': sign = 1;
    case '+': c = *psz++;
  }

//      Recognition of radix.

  if (radix == 0)
  {
    radix = 10;
    if (c == '0')
    {
      c = *psz++; radix = 8;
      if (c == 'X' || c == 'x') { c = *psz++; radix = 16; }
    }
  }

//      Find end of number.

  SqrTab (radix);                       // Filling of chartonum.
  int width = 0;
  for (;;)
  {
    if ((size_t)chartonum [c] >= radix) break;
    c = psz [width++];                  // Next symbol.
  }
  --psz;

//      Decoding of number.

  setunsign (EXPTRTO(char,psz,width), radix, width);

//      Assigning of sign.

  if (sign) this->neg();

  return *this;
}

//================================================
//      Conversion of unlimited number to string.
//================================================

//      Conversion of unlimited number to string for radixes 2..16,
//      sign and radix at left, number at right.
//
//      Format flags:
//      cBigNumber_unsign     unsigned output.
//      cBigNumber_uppercase  upper-case letters for number with radix > 10.
//      cBigNumber_showbase   add indicator of radix (0 or 0x).
//      cBigNumber_showpos    show sign for positive numbers.
//
//      Method returns pointer to static string which is
//      overwritten after each consequent call of method.

char*   cBigNumber::toatmp (            // Conversion to string.
                unsigned radix,         // Radix (2..16) or 0 (=10).
                unsigned width,         // Minimal number of chars in string.
                long     fill           // Fill-in character and format flags.
        ) const                         // Returns pointer to string
{                                       // in internal static buffer.
  assert (radix != 1);
  assert (radix <= MAX_RADIX);
  if (radix == 0) radix = 10;
  static exarray<char> str_;                    // Static buffer.
  size_t i = 0;                                 // Index in buffer.
  num_ = *this; num_.fit();                     // Normalization.
  if (fill & cBigNumber_unsign) num_.unsign();  // Make the number unsigned.

//      Select conversion table.

  register const EXPTR(char) ps = EXPTRTO (char,
                    numtochar [(fill & cBigNumber_uppercase) != 0],
                    sizeof (numtochar [0]));

//      Output of sign and prefixes.

  {
    long sign = num_.comp0();                   // Sign.

    if (sign < 0)
    {
      str_ [i++] = '-'; num_.neg();             // Output sign '-'.
    }
    else if (sign > 0 && (fill & cBigNumber_showpos))
    {
      str_ [i++] = '+';                         // Output sign '+'.
    }

    if (radix % 8 == 0 && (fill & cBigNumber_showbase))
    {
      str_ [i++] = '0';                         // Output radix.
      if (radix == 16) str_ [i++] = ps [16];
    }
  }

//      Output of number in reverse order.
//
//      Initial (non-optimized algorithm looks as the following:
//      do
//      {
//        str_ [i++] = ps [num % radix];
//        num_ /= radix;
//      }
//      while (num_ != 0);
//
//      In optimized algorithm we use method setdivmodtab for
//      division with module. To reduce number of long divisions
//      we use division to coefficients powradix, obtained by
//      consequent squaring of radix.
//
//      As a result of division to the highest coefficient with index
//      npmax we obtain quotient and module, which are divided to
//      coefficient with index npmax-1, e.t.c, until as result
//      of division to lowest coefficient witc index 0 we obtain
//      number with fit to digital grid of type long. Long word
//      is converted to string be consequent division to radix.
//
//      The highest quotient is stored in num_. Lower quotients are stored
//      in the stack of quotients div and marked in the stack of flags kdiv.
//      Flag is 1, if the corresponding div contains low quotient.
//      Initially array of quotients is empty. For the highest quotient
//      the corresponding kdiv is always 0.
//
//      ALGORITHM
//      When the highest quotient is divided quotient is stored in num_
//      and module is stored in div [npmax - 1]. When div [npmax - 1]
//      is divided quotient is stored in div [npmax - 1] and module is
//      stored in div [npmax - 2] e.t.c. Short module and quotient from
//      division of div [0] are coverted to string. While module is
//      converted quotient is stored in div [0]. Then quotient div [1]
//      is moved to div [0] and conversion is repeated, starting from
//      div [0], then div [2] is moved to div [1] and the process is
//      repeated from div [1], e.t.c. np to the lowers quotient, when
//      npmax is decreased and the algorithm is repeated once again.

  size_t j = i;                                 // Position of the 1st digit.
  {
//      Filling of stack of coefficients powradix [radix].
//      Required number of coefficients is returned.

    size_t np = SqrTab (radix, num_.length(), MAX_NP_OUT);
    EXPTR(cBigNumber) powr = EXPTRTYPE(powradix [radix]);

//      Work stacks.

    EXPTR(cBigNumber)  div = EXPTRTYPE(stk_);   // Stack of quotients.
    EXPTR(int)        kdiv = EXPTRTYPE(kstk_);  // Stack of flags.

//      Progress indication.

    cTTY cProgress (cBigNumberProgress);
    size_t numlength = num_.length() * sizeof (long);
    if (numlength < cTTY_numscaleK) numlength = 0;

//      Cycle of quotients.
//      All parts if number are converted, except for the last module

    size_t npmax = np;                          // Index of highest quotient.
    for (;;)                                    // Cycle of quotient.
    {
      if (np == npmax)                          // The highest quotient.
      {
        if (numlength != 0 && np == MAX_NP_OUT)
          cProgress.showScaled (cBigNumber::pszFormatting,
            (long)(numlength - num_.length() * sizeof (long)),
            (long)numlength);

//        The highest quotient is divided to the corresponding coefficient.
//        If result is 0 and index of the highest quotient os greater 0
//        then index of the highest quotient decreases and the cycle
//        repeats once again.

        num_.swap (mod_);                       // Store quotient to num_
        num_.setdivmodtab (mod_, powr [np]);    // and module to mod_.

        if (num_.comp0() == 0)                  // Highest quotient is over,
        {                                       // highest module remains.
          if (np == 0) break;                   // If it is the last module
                                                // the cycle is over,
          num_.swap (mod_);                     // else module becomes
          npmax = --np;                         // the highest quotient,
          numlength = 0;                        // indication is blocked
          assert (kdiv [np] == 0);              // and after debug check
          continue;                             // division is repeated.
        }
      }
      else                                      // Find of quotient.
      {
        assert (np > 0);                        // Debug check.
        assert (kdiv [np - 1] == 0);            // Debug check.
        if (kdiv [np] == 0)                     // If no quotient is found,
        {                                       // then go to higher
          ++np; continue;                       // quotient.
        }                                       // If quotient is found,
        mod_.swap (div [np]);                   // then load div [np]
        kdiv [np] = 0;                          // and mark is as free.
      }

//      Cycle of division of module - which index os greater then 0,
//      divide it to the corresponding coefficient.

      for (; np != 0; np--)
      {
        assert (kdiv [np - 1] == 0);            // Debug check.
        kdiv [np - 1] = 1;                      // Set flag.
        div  [np - 1].setdivmodtab (mod_, powr [np - 1]);
                                                // Store quotient to div
      }                                         // and module to mod_.

//      Cycle of conversion of the lowest module and quotient,
//      which fits to digital grid of type long.
//      The lowest module is stored in mod_ and the lowest quotient,
//      is stored in div [0], if it exists there (it is always so,
//      except for case npmax == 0).

      {
        register long lnum;                     // Integer.
        lnum = mod_.loword();                   // Output of module.
        for (;;)                                // Repeat cycle
        {                                       // once or twice.
          register long knum = maxradix [radix] [1] - 1;
          do                                    // Output knum digits.
          {
            str_ [i++] = ps [ (int) (lnum % radix) ];
            lnum /= radix;                      // Output digits.
          }
          while (--knum != 0);

          assert (lnum >= 0);                   // Debug check.
          assert (lnum < (long) radix);         // Debug check.
          str_ [i++] = ps [(int)(lnum)];        // The last digit.

          if (kdiv [0] == 0) break;             // Is there quotient?
          lnum = div [0].loword();              // Output of quotient.
          kdiv [0] = 0;                         // div [0] is free.
        }
      }

//      Repeat the cycle, starting form the most lower quotient.

      np = (npmax != 0);
    }                                           // Cycle on quotients.

//      Output the last module.

    {
      long lnum = mod_.loword();
      if (lnum != 0)                            // Short module.
      do
      {
        str_ [i++] = ps [(int) (lnum % radix)]; // Output of digit.
      }
      while ((lnum /= radix) != 0);
    }

//      Output 0 for number 0.

    if (i == j) str_ [i++] = ps [0];            // The number is 0.
  }

//      Output of fill-in character and finishing of string.

  {
    if ((char)fill == '\0') fill = ps [0];      // Determine fill-in character.
    while (i < width) str_ [i++] = (char)fill;  // Output of fill-in character.
    str_ [i--] = '\0';                          // String is ready.
  }

//      Swapping of symbols.

  {
    register EXPTR(char) ps = EXPTRTYPE(str_);
    while (i > j)
    {
      char c = ps [j]; ps [j] = ps [i]; j++; ps [i] = c; i--;
    }
  }

//      Conversion is finished.

  EXDEBUG (num_ = 0);                           // Debug clearing.
  EXDEBUG (mod_ = 0);                           // Debug clearing.
  return ((char*)(const char *)str_);           // Return string.
}

//================================================
//      Implementation of console output.
//================================================

cTTY& operator << (cTTY& os, const cBigNumber& b)
{
  cTTY_h ph = os;
  if (ph) (*ph) (&os, b.toatmp(), 0, 0, 0);
  return os;
}

void operator <<= (cTTY& os, const cBigNumber& b)
{
  cTTY_h ph = os;
  if (ph) (*ph) (&os, b.toatmp(), 0, 0, '\r');
}

//================================================
//      Hexadecimal dump of unsigned long value.
//================================================

cHexDump::cHexDump (unsigned long num)
{
  buffer [BITS / 4] = 0;
  for (size_t i = BITS / 4; i > 0; i--)
  {
    buffer [i - 1] = cBigNumber::numtochar [0] [(size_t)num & 0xF];
    num >>= 4;
  }
}
